import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _possibleConstructorReturn from "@babel/runtime/helpers/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/getPrototypeOf";
import _inherits from "@babel/runtime/helpers/inherits";
import { AttributeType, gl } from '@antv/l7-core';
import BaseLayer from '../core/BaseLayer';
import { RasterImageTriangulation } from '../core/triangulation';
var rasterImageFrag = "precision mediump float;\nuniform float u_opacity: 1.0;\nuniform sampler2D u_texture;\nvarying vec2 v_texCoord;\nvoid main() {\n  vec4 color = texture2D(u_texture,vec2(v_texCoord.x,v_texCoord.y));\n  gl_FragColor = color;\n}\n";
var rasterImageVert = "precision highp float;\nuniform mat4 u_ModelMatrix;\nattribute vec3 a_Position;\nattribute vec2 a_Uv;\nvarying vec2 v_texCoord;\n#pragma include \"projection\"\nvoid main() {\n   v_texCoord = a_Uv;\n   vec4 project_pos = project_position(vec4(a_Position, 1.0));\n   gl_Position = project_common_position_to_clipspace(vec4(project_pos.xy,0., 1.0));\n}\n";

var ImageLayer = function (_BaseLayer) {
  _inherits(ImageLayer, _BaseLayer);

  function ImageLayer() {
    var _getPrototypeOf2;

    var _this;

    _classCallCheck(this, ImageLayer);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _possibleConstructorReturn(this, (_getPrototypeOf2 = _getPrototypeOf(ImageLayer)).call.apply(_getPrototypeOf2, [this].concat(args)));
    _this.type = 'ImageLayer';
    _this.texture = void 0;
    return _this;
  }

  _createClass(ImageLayer, [{
    key: "getConfigSchema",
    value: function getConfigSchema() {
      return {
        properties: {
          opacity: {
            type: 'number',
            minimum: 0,
            maximum: 1
          }
        }
      };
    }
  }, {
    key: "renderModels",
    value: function renderModels() {
      var _this2 = this;

      var _this$getLayerConfig = this.getLayerConfig(),
          opacity = _this$getLayerConfig.opacity;

      if (this.texture) {
        this.models.forEach(function (model) {
          return model.draw({
            uniforms: {
              u_opacity: opacity || 1,
              u_texture: _this2.texture
            }
          });
        });
      }

      return this;
    }
  }, {
    key: "buildModels",
    value: function buildModels() {
      var _this3 = this;

      this.registerBuiltinAttributes();
      var source = this.getSource();
      var createTexture2D = this.rendererService.createTexture2D;
      source.data.images.then(function (imageData) {
        _this3.texture = createTexture2D({
          data: imageData[0],
          width: imageData[0].width,
          height: imageData[0].height
        });

        _this3.renderModels();
      });
      this.models = [this.buildLayerModel({
        moduleName: 'RasterImage',
        vertexShader: rasterImageVert,
        fragmentShader: rasterImageFrag,
        triangulation: RasterImageTriangulation,
        primitive: gl.TRIANGLES,
        depth: {
          enable: false
        },
        blend: {
          enable: true,
          func: {
            srcRGB: gl.SRC_ALPHA,
            srcAlpha: 1,
            dstRGB: gl.ONE_MINUS_SRC_ALPHA,
            dstAlpha: 1
          }
        }
      })];
    }
  }, {
    key: "registerBuiltinAttributes",
    value: function registerBuiltinAttributes() {
      this.styleAttributeService.registerStyleAttribute({
        name: 'uv',
        type: AttributeType.Attribute,
        descriptor: {
          name: 'a_Uv',
          buffer: {
            usage: gl.DYNAMIC_DRAW,
            data: [],
            type: gl.FLOAT
          },
          size: 2,
          update: function update(feature, featureIdx, vertex, attributeIdx) {
            return [vertex[3], vertex[4]];
          }
        }
      });
    }
  }]);

  return ImageLayer;
}(BaseLayer);

export { ImageLayer as default };
//# sourceMappingURL=image.js.map